@@ -9,12 +9,13 @@ import os
 import re
 import tempfile
 
-from mercurial import ui, util, error, scmutil, phases
+from mercurial import util, error, scmutil, phases
 
-from tortoisehg.util import hglib, shlib, wconfig, hgversion
+from tortoisehg.util import hglib, shlib, wconfig
 
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt.messageentry import MessageEntry
+from tortoisehg.hgqt import thgrepo
 from tortoisehg.hgqt import qtlib, qscilib, status, cmdui, branchop, revpanel
 from tortoisehg.hgqt import hgrcutil, mqutil, lfprompt, i18n, partialcommit
 
@@ -74,8 +75,6 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
     progress = pyqtSignal(QString, object, QString, QString, object)
     output = pyqtSignal(QString, QString)
     makeLogVisible = pyqtSignal(bool)
-    beginSuppressPrompt = pyqtSignal()
-    endSuppressPrompt = pyqtSignal()
 
     def __init__(self, repo, pats, opts, embedded=False, parent=None, rev=None):
         QWidget.__init__(self, parent)
@@ -86,7 +85,10 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.repo = repo
         self._rev = rev
         self.lastAction = None
-        self.lastCommitMsg = ''
+        # Dictionary storing the last "commit messages"
+        # 'commit' is used for 'commit' and 'qnew', while
+        # 'amend' is used for 'amend' and 'qrefresh'
+        self.lastCommitMsgs = {'commit': '', 'amend': ''}
         self.currentAction = None
         self.currentProgress = None
 
@@ -106,8 +108,6 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.runner.output.connect(self.output)
         self.runner.progress.connect(self.progress)
         self.runner.makeLogVisible.connect(self.makeLogVisible)
-        self.runner.commandStarted.connect(self.beginSuppressPrompt)
-        self.runner.commandFinished.connect(self.endSuppressPrompt)
         self.runner.commandFinished.connect(self.commandFinished)
 
         layout = QVBoxLayout()
@@ -288,14 +288,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             if ispatch(r):
                 return False
             ctx = r.changectx('.')
-            canamendctx = (ctx.phase() != phases.public) \
+            return (ctx.phase() != phases.public) \
                 and len(r.changectx(None).parents()) < 2 \
                 and not ctx.children()
-            # hg < 2.6
-            if hgversion.hgversion < '2.6':
-                canamendctx = canamendctx \
-                    and len(ctx.parents()) < 2
-            return canamendctx
 
         acts = [
             ('commit', _('Commit changes'), _('Commit'), notpatch),
@@ -408,11 +403,20 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 allowcs = len(self.repo.parents()) == 1
         if curraction._name in ('qref', 'amend'):
             if self.lastAction not in ('qref', 'amend'):
-                self.lastCommitMsg = self.msgte.text()
-            self.setMessage(hglib.tounicode(pctx.description()))
+                self.lastCommitMsgs['commit'] = self.msgte.text()
+            if self.lastCommitMsgs['amend']:
+                msg = self.lastCommitMsgs['amend']
+            else:
+                msg = hglib.tounicode(pctx.description())
+            self.setMessage(msg)
         else:
             if self.lastAction in ('qref', 'amend'):
-                self.setMessage(self.lastCommitMsg)
+                self.lastCommitMsgs['amend'] = self.msgte.text()
+                self.setMessage(self.lastCommitMsgs['commit'])
+        if curraction._name == 'amend':
+            self.stwidget.defcheck = 'amend'
+        else:
+            self.stwidget.defcheck = 'commit'
         self.stwidget.fileview.enableChangeSelection(allowcs)
         if not allowcs:
             self.stwidget.partials = {}
@@ -421,7 +425,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.committb.setText(curraction._text)
         self.lastAction = curraction._name
 
-    def getBranchCommandLine(self, branchName, repo):
+    def getBranchCommandLine(self):
         '''
         Create the command line to change or create the selected branch unless
         it is the selected branch
@@ -433,6 +437,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         the selected action
         '''
         # This function is used both by commit() and mqPerformAction()
+        repo = self.repo
         commandlines = []
         newbranch = False
         branch = hglib.fromunicode(self.branchop)
@@ -477,8 +482,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         # Check if we need to change branch first
         wholecmdlines = []  # [[cmd1, ...], [cmd2, ...], ...]
         if self.branchop:
-            cmdlines, newbranch = self.getBranchCommandLine(self.branchop,
-                                                            self.repo)
+            cmdlines, newbranch = self.getBranchCommandLine()
             if cmdlines is None:
                 return
             wholecmdlines.extend(cmdlines)
@@ -581,6 +585,12 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
     @pyqtSlot()
     def repositoryChanged(self):
         'Repository has detected a changelog / dirstate change'
+        # clear the last 'amend' message
+        # do not clear the last 'commit' message because there are many cases
+        # in which we may write a commit message first, modify the repository
+        # (e.g. amend or update and merge uncommitted changes) and then do the
+        # actual commit
+        self.lastCommitMsgs['amend'] = ''
         self.refresh()
         self.stwidget.refreshWctx() # Trigger reload of working context
 
@@ -678,6 +688,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                                  QMessageBox.Ok | QMessageBox.Cancel)
         if d != QMessageBox.Ok:
             return
+        self.repo.incrementBusyCount()
         self.currentAction = 'rollback'
         self.currentProgress = _('Rollback', 'start progress')
         self.progress.emit(*cmdui.startProgress(self.currentProgress, ''))
@@ -736,7 +747,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.msghistory = list(s.value(gpref+'history-'+repoid).toStringList())
         self.msghistory = [unicode(m) for m in self.msghistory if m]
         self.updateRecentMessages()
-        self.userhist = s.value(gpref+'userhist').toStringList()
+        self.userhist = map(unicode, s.value(gpref+'userhist').toStringList())
         self.userhist = [u for u in self.userhist if u]
         try:
             curmsg = self.repo.opener('cur-message.txt').read()
@@ -831,8 +842,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         elif self.branchop == False:
             brcmd = ['--close-branch']
         else:
-            commandlines, newbranch = self.getBranchCommandLine(self.branchop,
-                                                                self.repo)
+            commandlines, newbranch = self.getBranchCommandLine()
             if commandlines is None:
                 return
         partials = []
@@ -995,7 +1005,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.branchop = None
             umsg = self.msgte.text()
             if self.currentAction not in ('qref', 'amend'):
-                self.lastCommitMsg = ''
+                self.lastCommitMsgs['commit'] = ''
                 if self.currentAction == 'commit':
                     # capture last message for BugTraq plugin
                     self.lastmessage = self.getMessage(True)
@@ -1348,6 +1358,9 @@ class CommitDialog(QDialog):
         toplayout.addWidget(self.bb)
         layout.addWidget(self.statusbar)
 
+        self._subdialogs = qtlib.DialogKeeper(CommitDialog._createSubDialog,
+                                              parent=self)
+
         s = QSettings()
         self.restoreGeometry(s.value('commit/geom').toByteArray())
         commit.loadSettings(s, 'committool')
@@ -1362,10 +1375,13 @@ class CommitDialog(QDialog):
         qtlib.newshortcutsforstdkey(QKeySequence.Refresh, self, self.refresh)
 
     def linkActivated(self, link):
-        link = hglib.fromunicode(link)
+        link = unicode(link)
         if link.startswith('repo:'):
-            from tortoisehg.hgqt.run import qtrun
-            qtrun(run, ui.ui(), root=link[len('repo:'):])
+            self._subdialogs.open(link[len('repo:'):])
+
+    def _createSubDialog(self, uroot):
+        repo = thgrepo.repository(None, hglib.fromunicode(uroot))
+        return CommitDialog(repo, [], {}, parent=self)
 
     @pyqtSlot()
     def updateUndo(self):
@@ -1409,12 +1425,3 @@ class CommitDialog(QDialog):
     def reject(self):
         if self.promptExit():
             QDialog.reject(self)
-
-def run(ui, *pats, **opts):
-    from tortoisehg.util import paths
-    from tortoisehg.hgqt import thgrepo
-    root = opts.get('root', paths.find_root())
-    repo = thgrepo.repository(ui, path=root)
-    pats = hglib.canonpaths(pats)
-    os.chdir(repo.root)
-    return CommitDialog(repo, pats, opts)
