@@ -19,7 +19,8 @@ from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, thgrepo
 
 PANEL_DEFAULT = ('rev', 'summary', 'user', 'dateage', 'branch', 'close',
-                 'tags', 'transplant', 'p4', 'svn', 'converted')
+                 'tags', 'graft', 'transplant', 'obsolete',
+                 'p4', 'svn', 'converted',)
 
 def create(repo, target=None, style=None, custom=None, **kargs):
     return Factory(repo, custom, style, target, **kargs)()
@@ -108,7 +109,9 @@ class SummaryInfo(object):
               'dateage': _('Date:'), 'branch': _('Branch:'),
               'close': _('Close:'),
               'tags': _('Tags:'), 'rawbranch': _('Branch:'),
-              'rawtags': _('Tags:'), 'transplant': _('Transplant:'),
+              'rawtags': _('Tags:'), 'graft': _('Graft:'),
+              'transplant': _('Transplant:'),
+              'obsolete': _('Obsolete state:'),
               'p4': _('Perforce:'), 'svn': _('Subversion:'),
               'converted': _('Converted From:'), 'shortuser': _('User:')}
 
@@ -133,10 +136,11 @@ class SummaryInfo(object):
             elif item == 'desc':
                 return hglib.tounicode(ctx.description().replace('\0', ''))
             elif item == 'summary':
-                value = ctx.description().replace('\0', '').split('\n')[0]
-                if len(value) == 0:
+                summary = hglib.longsummary(
+                    ctx.description().replace('\0', ''))
+                if len(summary) == 0:
                     return None
-                return hglib.tounicode(value)[:80]
+                return summary
             elif item == 'user':
                 user = hglib.user(ctx)
                 if user:
@@ -178,6 +182,13 @@ class SummaryInfo(object):
                 return hglib.getrawctxtags(ctx)
             elif item == 'tags':
                 return hglib.getctxtags(ctx)
+            elif item == 'graft':
+                extra = ctx.extra()
+                try:
+                    return extra['source']
+                except KeyError:
+                    pass
+                return None
             elif item == 'transplant':
                 extra = ctx.extra()
                 try:
@@ -187,6 +198,16 @@ class SummaryInfo(object):
                 except KeyError:
                     pass
                 return None
+            elif item == 'obsolete':
+                obsoletestate = []
+                if ctx.obsolete():
+                    obsoletestate.append('obsolete')
+                if ctx.extinct():
+                    obsoletestate.append('extinct')
+                obsoletestate += ctx.troubles()
+                if obsoletestate:
+                    return obsoletestate
+                return None
             elif item == 'p4':
                 extra = ctx.extra()
                 p4cl = extra.get('p4', None)
@@ -254,7 +275,7 @@ class SummaryInfo(object):
                 if revnum is not None and revid is not None:
                     return '%s (%s)' % (revnum, revid)
                 return '%s' % revid
-            elif item in ('revid', 'transplant'):
+            elif item in ('revid', 'graft', 'transplant'):
                 return qtlib.markup(value, **mono)
             elif item in ('revnum', 'p4', 'close', 'converted'):
                 return str(value)
@@ -273,6 +294,11 @@ class SummaryInfo(object):
                 return qtlib.markup(value)
             elif item == 'dateage':
                 return qtlib.markup('%s (%s)' % value)
+            elif item == 'obsolete':
+                opts = dict(fg='black', bg='#ff8566')
+                obsoletestates = [qtlib.markup(' %s ' % state, **opts)
+                                  for state in value]
+                return ' '.join(obsoletestates)
             raise UnknownItem(item)
         value = self.get_data(item, *args)
         if value is None:
@@ -379,16 +405,15 @@ class SummaryPanel(SummaryBase, QWidget):
 
         if self.revlabel is None:
             self.revlabel = QLabel()
-            self.revlabel.linkActivated.connect(
-                 lambda s: self.linkActivated.emit(s))
-            self.layout().addWidget(self.revlabel, alignment=Qt.AlignTop)
+            self.revlabel.linkActivated.connect(self.linkActivated)
+            self.layout().addWidget(self.revlabel, 0, Qt.AlignTop)
 
         if 'expandable' in self.csstyle and self.csstyle['expandable']:
             if self.expand_btn.parentWidget() is None:
                 self.expand_btn.clicked.connect(lambda: self.update())
                 margin = QHBoxLayout()
                 margin.setMargin(3)
-                margin.addWidget(self.expand_btn, alignment=Qt.AlignTop)
+                margin.addWidget(self.expand_btn, 0, Qt.AlignTop)
                 self.layout().insertLayout(0, margin)
             self.expand_btn.setShown(True)
         elif self.expand_btn.parentWidget() is not None:
