@@ -36,17 +36,20 @@ otherside = {'left': 'right', 'right': 'left'}
 class _AbstractFileDialog(QMainWindow):
     finished = pyqtSignal(int)
 
-    def __init__(self, repo, filename, repoviewer=None):
+    def __init__(self, repo, filename):
         QMainWindow.__init__(self)
         self.repo = repo
 
         self.setupUi(self)
-        self.setRepoViewer(repoviewer)
         self._show_rev = None
 
         assert not isinstance(filename, (unicode, QString))
         self.filename = filename
 
+        self.setWindowTitle(_('Hg file log viewer [%s] - %s')
+                            % (repo.displayname, hglib.tounicode(filename)))
+        self.setWindowIcon(qtlib.geticon('hg-log'))
+
         self.createActions()
         self.setupToolbars()
 
@@ -57,15 +60,6 @@ class _AbstractFileDialog(QMainWindow):
         super(_AbstractFileDialog, self).closeEvent(event)
         self.finished.emit(0)  # mimic QDialog exit
 
-    def setRepoViewer(self, repoviewer=None):
-        self.repoviewer = repoviewer
-        if repoviewer:
-            repoviewer.finished.connect(self._clearRepoViewer)
-
-    @pyqtSlot()
-    def _clearRepoViewer(self):
-        self.setRepoViewer(None)
-
     def reload(self):
         'Reload toolbar action handler'
         self.repo.thginvalidate()
@@ -75,22 +69,16 @@ class _AbstractFileDialog(QMainWindow):
         """
         Callback called when a revision is double-clicked in the revisions table
         """
-        if self.repoviewer is None:
-            # prevent recursive import
-            from workbench import Workbench
-            self.repoviewer = Workbench()
-        self.repoviewer.show()
-        self.repoviewer.activateWindow()
-        self.repoviewer.raise_()
-        self.repoviewer.showRepo(hglib.tounicode(self.repo.root))
-        self.repoviewer.goto(self.repo.root, rev)
+        # TODO: implement by using signal-slot if possible
+        from tortoisehg.hgqt import run
+        run.qtrun.showRepoInWorkbench(hglib.tounicode(self.repo.root), rev)
 
 class FileLogDialog(_AbstractFileDialog):
     """
     A dialog showing a revision graph for a file.
     """
-    def __init__(self, repo, filename, repoviewer=None):
-        super(FileLogDialog, self).__init__(repo, filename, repoviewer)
+    def __init__(self, repo, filename):
+        super(FileLogDialog, self).__init__(repo, filename)
         self._readSettings()
         self.menu = None
         self.dualmenu = None
@@ -196,8 +184,10 @@ class FileLogDialog(_AbstractFileDialog):
     def modelFilled(self):
         self.repoview.resizeColumns()
         if self._show_rev is not None:
-            index = self.filerevmodel.indexFromRev(self._show_rev)
+            index = self.filerevmodel.indexLinkedFromRev(self._show_rev)
             self._show_rev = None
+        elif self.repoview.currentIndex().isValid():
+            return  # already set by goto()
         else:
             index = self.filerevmodel.index(0,0)
         if index is not None:
@@ -259,14 +249,12 @@ class FileLogDialog(_AbstractFileDialog):
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, [], opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onVisualDiffToLocal(self):
         opts = dict(rev=['rev(%d)' % self.selection[0]])
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, [], opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onVisualDiffRevs(self):
         revs = self.selection
@@ -277,7 +265,6 @@ class FileLogDialog(_AbstractFileDialog):
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, [], opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onVisualDiffFile(self):
         rev = self.selection[0]
@@ -286,7 +273,6 @@ class FileLogDialog(_AbstractFileDialog):
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onVisualDiffFileToLocal(self):
         rev = self.selection[0]
@@ -295,7 +281,6 @@ class FileLogDialog(_AbstractFileDialog):
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onVisualDiffFileRevs(self):
         revs = self.selection
@@ -307,7 +292,6 @@ class FileLogDialog(_AbstractFileDialog):
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onEditLocal(self):
         filenames = [self.filename]
@@ -390,12 +374,24 @@ class FileLogDialog(_AbstractFileDialog):
         self.textView.showMessage.emit(msg)
 
     def goto(self, rev):
-        index = self.filerevmodel.indexFromRev(rev)
+        index = self.filerevmodel.indexLinkedFromRev(rev)
         if index is not None:
             self.repoview.setCurrentIndex(index)
         else:
             self._show_rev = rev
 
+    def showLine(self, line):
+        self.textView.showLine(line - 1)  # fileview should do -1 instead?
+
+    def setFileViewMode(self, mode):
+        self.textView.setMode(mode)
+
+    def setSearchPattern(self, text):
+        self.textView.searchbar.setPattern(text)
+
+    def setSearchCaseInsensitive(self, ignorecase):
+        self.textView.searchbar.setCaseInsensitive(ignorecase)
+
     def reload(self):
         self.repoview.saveSettings()
         super(FileLogDialog, self).reload()
@@ -404,8 +400,8 @@ class FileDiffDialog(_AbstractFileDialog):
     """
     Qt4 dialog to display diffs between different mercurial revisions of a file.
     """
-    def __init__(self, repo, filename, repoviewer=None):
-        super(FileDiffDialog, self).__init__(repo, filename, repoviewer)
+    def __init__(self, repo, filename):
+        super(FileDiffDialog, self).__init__(repo, filename)
         self._readSettings()
         self.menu = None
 
@@ -491,6 +487,7 @@ class FileDiffDialog(_AbstractFileDialog):
 
         for side, idx  in (('left', 0), ('right', 3)):
             sci = Scintilla(self.frame)
+            sci.installEventFilter(self)
             sci.verticalScrollBar().setFocusPolicy(Qt.StrongFocus)
             sci.setFocusProxy(sci.verticalScrollBar())
             sci.verticalScrollBar().installEventFilter(self)
@@ -552,6 +549,10 @@ class FileDiffDialog(_AbstractFileDialog):
         self.setTabOrder(table, self.viewers['left'])
         self.setTabOrder(self.viewers['left'], self.viewers['right'])
 
+        # timer used to merge requests of syncPageStep on ResizeEvent
+        self._delayedSyncPageStep = QTimer(self, interval=0, singleShot=True)
+        self._delayedSyncPageStep.timeout.connect(self.diffblock.syncPageStep)
+
         # timer used to fill viewers with diff block markers during GUI idle time
         self.timer = QTimer()
         self.timer.setSingleShot(False)
@@ -598,9 +599,21 @@ class FileDiffDialog(_AbstractFileDialog):
         if self._show_rev is not None:
             self.goto(self._show_rev)
             self._show_rev = None
+        elif self.tableView_revisions_left.currentIndex().isValid():
+            return  # already set by goto()
         elif len(self.filerevmodel.graph):
             self.goto(self.filerevmodel.graph[0].rev)
 
+    def eventFilter(self, watched, event):
+        if watched in self.viewers.values():
+            # copy page steps to diffblock _after_ viewers are resized; resize
+            # events will be posted in arbitrary order.
+            if event.type() == QEvent.Resize:
+                self._delayedSyncPageStep.start()
+            return False
+        else:
+            return super(FileDiffDialog, self).eventFilter(watched, event)
+
     def onRevisionSelected(self, rev):
         if rev is None or rev not in self.filerevmodel.graph.nodesdict:
             return
@@ -615,7 +628,7 @@ class FileDiffDialog(_AbstractFileDialog):
         self.update_diff(keeppos=otherside[side])
 
     def goto(self, rev):
-        index = self.filerevmodel.indexFromRev(rev)
+        index = self.filerevmodel.indexLinkedFromRev(rev)
         if index is not None:
             if index.row() == 0:
                 index = self.filerevmodel.index(1, 0)
@@ -817,14 +830,12 @@ class FileDiffDialog(_AbstractFileDialog):
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, [], opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onVisualDiffToLocal(self):
         opts = dict(rev=['rev(%d)' % self.selection[0]])
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, [], opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onVisualDiffFile(self):
         rev = self.selection[0]
@@ -833,7 +844,6 @@ class FileDiffDialog(_AbstractFileDialog):
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onVisualDiffFileToLocal(self):
         rev = self.selection[0]
@@ -842,7 +852,6 @@ class FileDiffDialog(_AbstractFileDialog):
         dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
         if dlg:
             dlg.exec_()
-            dlg.deleteLater()
 
     def onEditLocal(self):
         filenames = [self.filename]
