@@ -122,6 +122,7 @@ class GraftDialog(QDialog):
         self.layout().addWidget(bbox)
         self.bbox = bbox
 
+        self.th = None
         if self.checkResolve():
             self.abortbtn.setEnabled(True)
         else:
@@ -301,6 +302,8 @@ class GraftDialog(QDialog):
             self.runner.run(cmdline)
 
     def reject(self):
+        if self.th and self.th.isRunning():
+            return
         if os.path.exists(self._graftstatefile):
             main = _('Exiting with an unfinished graft is not recommended.')
             text = _('Consider aborting the graft first.')
@@ -310,19 +313,3 @@ class GraftDialog(QDialog):
                                         labels=labels, parent=self):
                 return
         super(GraftDialog, self).reject()
-
-def run(ui, *revs, **opts):
-    from tortoisehg.util import paths
-    repo = thgrepo.repository(ui, path=paths.find_root())
-
-    revs = list(revs)
-    revs.extend(opts['rev'])
-
-    if os.path.exists(repo.join('graftstate')):
-        qtlib.InfoMsgBox(_('Graft already in progress'),
-                          _('Resuming graft already in progress'))
-    elif not revs:
-        qtlib.ErrorMsgBox(_('Abort'),
-                          _('You must provide revisions to graft'))
-        import sys; sys.exit()
-    return GraftDialog(repo, None, source=revs)
