@@ -16,26 +16,37 @@
 
 """helper functions and classes to ease hg revision graph building
 
-Based on graphlog's algorithm, with insipration stolen from TortoiseHg
+Based on graphlog's algorithm, with inspiration stolen from TortoiseHg
 revision grapher (now stolen back).
+
+The primary interface are the *_grapher functions, which are generators
+of Graph instances that describe a revision set graph. These generators
+are used by repomodel.py which renders them on a widget.
 """
 
 import time
 import os
 import itertools
 
-from mercurial import util, error
+from mercurial import repoview
 
-try:
-    from mercurial import repoview
-    _filterrevs = repoview.filterrevs
-except ImportError: # hg < 2.5
-    def _filterrevs(repo, filtername):
-        return []
+LINE_TYPE_PARENT = 0
+LINE_TYPE_GRAFT = 1
 
 def revision_grapher(repo, **opts):
     """incremental revision grapher
 
+    param repo       The repository
+    opt   start_rev  Tip-most revision of range to graph
+    opt   stop_rev   0-most revision of range to graph
+    opt   follow     True means graph only ancestors of start_rev
+    opt   revset     set of revisions to graph.
+                     If used, then start_rev, stop_rev, and follow is ignored
+    opt   branch     Only graph this branch
+    opt   allparents If set in addition to branch, then cset outside the
+                     branch that are ancestors to some cset inside the branch
+                     is also graphed
+
     This generator function walks through the revision history from
     revision start_rev to revision stop_rev (which must be less than
     or equal to start_rev) and for each revision emits tuples with the
@@ -44,22 +55,21 @@ def revision_grapher(repo, **opts):
       - current revision
       - column of the current node in the set of ongoing edges
       - color of the node (?)
-      - lines; a list of (col, next_col, color) indicating the edges between
-        the current row and the next row
+      - lines: a list of (col, next_col, color_no, line_type, children, parent)
+          children: tuple of revs which connected to top of this line.
+                    (or current rev if node is on the line.)
+          parent:   rev which connected to bottom of this line.
+        defining the edges between the current row and the next row
       - parent revisions of current revision
-
-    If follow is True, only generated the subtree from the start_rev head.
-
-    If branch is set, only generated the subtree for the given named branch.
-
-    If allparents is set, include the branch heads for the selected named
-    branch heads and all ancestors. If not set, include only the revisions
-    on the selected named branch.
     """
 
     revset = opts.get('revset', None)
     branch = opts.get('branch', None)
-    revhidden = _filterrevs(repo, 'visible')
+    showhidden = opts.get('showhidden', None)
+    if showhidden:
+        revhidden = []
+    else:
+        revhidden = repoview.filterrevs(repo, 'visible')
     if revset:
         start_rev = max(revset)
         stop_rev = min(revset)
@@ -75,17 +85,19 @@ def revision_grapher(repo, **opts):
 
     curr_rev = start_rev
     revs = []
-    rev_color = {}
-    nextcolor = 0
+    children = [()]
+    links = [] # smallest link type that applies
 
     if opts.get('allparents') or not branch:
         def getparents(ctx):
-            return [x.rev() for x in ctx.parents() if x]
+            return [x for x in ctx.parents() if x]
     else:
         def getparents(ctx):
-            return [x.rev() for x in ctx.parents() \
+            return [x for x in ctx.parents()
                     if x and x.branch() == branch]
 
+    rev_color = RevColorPalette(getparents)
+
     while curr_rev is None or curr_rev >= stop_rev:
         if hidden(curr_rev):
             curr_rev -= 1
@@ -93,7 +105,6 @@ def revision_grapher(repo, **opts):
 
         # Compute revs and next_revs.
         ctx = repo[curr_rev]
-        # Compute revs and next_revs.
         if curr_rev not in revs:
             if branch and ctx.branch() != branch:
                 if curr_rev is None:
@@ -108,53 +119,68 @@ def revision_grapher(repo, **opts):
                 curr_rev -= 1
                 continue
             revs.append(curr_rev)
-            rev_color[curr_rev] = curcolor = nextcolor
-            nextcolor += 1
-            p_revs = getparents(ctx)
-            while p_revs:
-                rev0 = p_revs[0]
-                if rev0 < stop_rev or rev0 in rev_color:
-                    break
-                rev_color[rev0] = curcolor
-                p_revs = getparents(repo[rev0])
+            links.append(LINE_TYPE_PARENT)
+            children.append(())
+            rev_color.addheadctx(ctx)
         curcolor = rev_color[curr_rev]
         rev_index = revs.index(curr_rev)
         next_revs = revs[:]
+        next_links = links[:]
+        next_children = children[:]
 
         # Add parents to next_revs.
-        parents = [p for p in getparents(ctx) if not hidden(p)]
+        parents = [(p.rev(), LINE_TYPE_PARENT) for p in getparents(ctx)
+                   if not hidden(p.rev())]
+        if 'source' in ctx.extra():
+            src_rev_str = ctx.extra()['source']
+            if src_rev_str in repo:
+                src_rev = repo[src_rev_str].rev()
+                if stop_rev <= src_rev < curr_rev and not hidden(src_rev):
+                    parents.append((src_rev, LINE_TYPE_GRAFT))
         parents_to_add = []
+        links_to_add = []
+        children_to_add = []
         if len(parents) > 1:
             preferred_color = None
         else:
             preferred_color = curcolor
-        for parent in parents:
+        for parent, link_type in parents:
             if parent not in next_revs:
                 parents_to_add.append(parent)
+                links_to_add.append(link_type)
+                children_to_add.append((curr_rev,))
                 if parent not in rev_color:
-                    if preferred_color:
-                        rev_color[parent] = preferred_color
-                        preferred_color = None
-                    else:
-                        rev_color[parent] = nextcolor
-                        nextcolor += 1
+                    rev_color.assigncolor(parent, preferred_color)
+                    preferred_color = None
+            else:
+                # Merging lines should have the most solid style
+                #  (= lowest style value)
+                i = next_revs.index(parent)
+                next_links[i] = min(next_links[i], link_type)
+                next_children[i] += (curr_rev,)
             preferred_color = None
 
         # parents_to_add.sort()
         next_revs[rev_index:rev_index + 1] = parents_to_add
+        next_links[rev_index:rev_index + 1] = links_to_add
+        next_children[rev_index:rev_index + 1] = children_to_add
 
         lines = []
         for i, rev in enumerate(revs):
             if rev in next_revs:
                 color = rev_color[rev]
-                lines.append( (i, next_revs.index(rev), color) )
+                lines.append((i, next_revs.index(rev), color, links[i],
+                              children[i], rev))
             elif rev == curr_rev:
-                for parent in parents:
+                for parent, link_type in parents:
                     color = rev_color[parent]
-                    lines.append( (i, next_revs.index(parent), color) )
+                    lines.append((i, next_revs.index(parent), color, link_type,
+                                  (curr_rev,), parent))
 
-        yield (curr_rev, rev_index, curcolor, lines, parents)
+        yield GraphNode(curr_rev, rev_index, curcolor, lines, parents)
         revs = next_revs
+        links = next_links
+        children = next_children
         if curr_rev is None:
             curr_rev = len(repo)
         else:
@@ -176,6 +202,7 @@ def filelog_grapher(repo, path):
     heads.remove(rev)
 
     revs = []
+    children = [()]
     rev_color = {}
     nextcolor = 0
     _paths = {}
@@ -185,40 +212,49 @@ def filelog_grapher(repo, path):
         if rev not in revs:
             revs.append(rev)
             rev_color[rev] = nextcolor ; nextcolor += 1
+            children.append(())
         curcolor = rev_color[rev]
         index = revs.index(rev)
         next_revs = revs[:]
+        next_children = children[:]
 
         # Add parents to next_revs
         fctx = repo.filectx(_paths.get(rev, path), changeid=rev)
         for pfctx in fctx.parents():
             _paths[pfctx.rev()] = pfctx.path()
-        parents = [pfctx.rev() for pfctx in fctx.parents()]# if f.path() == path]
+        parents = [pfctx.rev() for pfctx in fctx.parents()]
+                   # if f.path() == path]
         parents_to_add = []
+        children_to_add = []
         for parent in parents:
             if parent not in next_revs:
                 parents_to_add.append(parent)
+                children_to_add.append((rev,))
                 if len(parents) > 1:
                     rev_color[parent] = nextcolor ; nextcolor += 1
                 else:
                     rev_color[parent] = curcolor
         parents_to_add.sort()
         next_revs[index:index + 1] = parents_to_add
+        next_children[index:index + 1] = children_to_add
 
         lines = []
         for i, nrev in enumerate(revs):
             if nrev in next_revs:
                 color = rev_color[nrev]
-                lines.append( (i, next_revs.index(nrev), color) )
+                lines.append((i, next_revs.index(nrev), color, LINE_TYPE_PARENT,
+                              children[i], nrev))
             elif nrev == rev:
                 for parent in parents:
                     color = rev_color[parent]
-                    lines.append( (i, next_revs.index(parent), color) )
+                    lines.append((i, next_revs.index(parent), color,
+                                  LINE_TYPE_PARENT, (rev,), parent))
 
         pcrevs = [pfc.rev() for pfc in fctx.parents()]
-        yield (fctx.rev(), index, curcolor, lines, pcrevs,
-               _paths.get(fctx.rev(), path))
+        yield GraphNode(fctx.rev(), index, curcolor, lines, pcrevs,
+                        extra=[_paths.get(fctx.rev(), path)])
         revs = next_revs
+        children = next_children
 
         if revs:
             rev = max(revs)
@@ -230,11 +266,63 @@ def filelog_grapher(repo, path):
 def mq_patch_grapher(repo):
     """Graphs unapplied MQ patches"""
     for patchname in reversed(repo.thgmqunappliedpatches):
-        yield (patchname, 0, "", [], [], "")
+        yield GraphNode(patchname, 0, "", [], [])
+
+class RevColorPalette(object):
+    """Assign node and line colors for each revision"""
+
+    def __init__(self, getparents):
+        self._getparents = getparents
+        self._pendingheads = []
+        self._knowncolors = {}
+        self._nextcolor = 0
+
+    def addheadctx(self, ctx):
+        color = self.assigncolor(ctx.rev())
+        p_ctxs = self._getparents(ctx)
+        self._pendingheads.append((p_ctxs, color))
+
+    def _fillpendingheads(self, stoprev):
+        if stoprev is None:
+            return  # avoid filling everything (int_rev < None is False)
+
+        nextpendingheads = []
+        for p_ctxs, color in self._pendingheads:
+            pending = self._fillancestors(p_ctxs, color, stoprev)
+            if pending:
+                nextpendingheads.append((pending, color))
+        self._pendingheads = nextpendingheads
+
+    def _fillancestors(self, p_ctxs, curcolor, stoprev):
+        while p_ctxs:
+            ctx0 = p_ctxs[0]
+            rev0 = ctx0.rev()
+            if rev0 < stoprev:
+                return p_ctxs
+            if rev0 in self._knowncolors:
+                return
+            self._knowncolors[rev0] = curcolor
+            p_ctxs = self._getparents(ctx0)
+
+    def assigncolor(self, rev, color=None):
+        self._fillpendingheads(rev)
+        if color is None:
+            color = self._nextcolor
+            self._nextcolor += 1
+        self._knowncolors[rev] = color
+        return color
+
+    def __getitem__(self, rev):
+        self._fillpendingheads(rev)
+        return self._knowncolors[rev]
+
+    def __contains__(self, rev):
+        self._fillpendingheads(rev)
+        return rev in self._knowncolors
 
 class GraphNode(object):
     """
-    Simple class to encapsulate e hg node in the revision graph. Does
+    Simple class to encapsulate a hg node in the revision graph. Does
     nothing but declaring attributes.
     """
     def __init__(self, rev, xposition, color, lines, parents, ncols=None,
@@ -253,7 +341,7 @@ class GraphNode(object):
 class Graph(object):
     """
     Graph object to ease hg repo navigation. The Graph object
-    instanciate a `revision_grapher` generator, and provide a `fill`
+    instantiate a `revision_grapher` generator, and provide a `fill`
     method to build the graph progressively.
     """
     #@timeit
@@ -307,21 +395,18 @@ class Graph(object):
         stopped = False
         mcol = set([self.max_cols])
 
-        for vnext in self.grapher:
-            if vnext is None:
+        for gnode in self.grapher:
+            if gnode is None:
                 continue
-            nrev, xpos, color, lines, parents = vnext[:5]
-            if not type(nrev) == str and nrev >= self.maxlog:
+            if not type(gnode.rev) == str and gnode.rev >= self.maxlog:
                 continue
-            gnode = GraphNode(nrev, xpos, color, lines, parents,
-                              extra=vnext[5:])
             if self.nodes:
                 gnode.toplines = self.nodes[-1].bottomlines
             self.nodes.append(gnode)
-            self.nodesdict[nrev] = gnode
-            mcol = mcol.union(set([xpos]))
+            self.nodesdict[gnode.rev] = gnode
+            mcol = mcol.union(set([gnode.x]))
             mcol = mcol.union(set([max(x[:2]) for x in gnode.bottomlines]))
-            if rev is not None and nrev <= rev:
+            if rev is not None and gnode.rev <= rev:
                 rev = None # we reached rev, switching to nnode counter
             if rev is None:
                 if nnodes is not None:
