@@ -230,7 +230,6 @@ class MQPatchesWidget(QDockWidget):
             return cont
         if checkGuardsOrComments():
             dlg = qreorder.QReorderDialog(self.repo, self)
-            dlg.finished.connect(dlg.deleteLater)
             dlg.exec_()
 
     @pyqtSlot()
@@ -272,7 +271,6 @@ class MQPatchesWidget(QDockWidget):
         from tortoisehg.hgqt import qdelete
         patch = self.queueListWidget.currentItem()._thgpatch
         dlg = qdelete.QDeleteDialog(self.repo, [patch], self)
-        dlg.finished.connect(dlg.deleteLater)
         if dlg.exec_() == QDialog.Accepted:
             self.reload()
 
@@ -353,6 +351,7 @@ class MQPatchesWidget(QDockWidget):
         self.queueListWidget.clear()
 
         if self.repo is None:
+            self.guardSelBtn.setEnabled(False)
             self.qpushAllAct.setEnabled(False)
             self.qpushAct.setEnabled(False)
             self.qdeleteAct.setEnabled(False)
@@ -630,6 +629,14 @@ class MQWidget(QWidget, qtlib.TaskWidget):
         self.finishfunc = self.checkForRejects
         self.cmd.run(cmdline)
 
+    def popAll(self):
+        if self.cmd.running():
+            return
+        self.repo.incrementBusyCount()
+        cmdline = ['qpop', '-R', self.repo.root, '--all']
+        cmdline += self.getUserOptions('force')
+        self.cmd.run(cmdline)
+
     @pyqtSlot(int)
     def onMessageSelected(self, row):
         if self.messageEditor.text() and self.messageEditor.isModified():
@@ -836,7 +843,6 @@ class MQWidget(QWidget, qtlib.TaskWidget):
             super(MQWidget, self).dropEvent(event)
             return
         dlg = thgimport.ImportDialog(self.repo, self, mq=True)
-        dlg.finished.connect(dlg.deleteLater)
         dlg.setfilepaths(patches)
         dlg.exec_()
 
@@ -973,10 +979,3 @@ class OptionsDialog(QDialog):
 
         self.outopts = outopts
         QDialog.accept(self)
-
-
-
-def run(ui, *pats, **opts):
-    from tortoisehg.util import paths
-    repo = thgrepo.repository(ui, path=paths.find_root())
-    return MQWidget(repo, None, **opts)
