@@ -6,11 +6,8 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import os
-
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
-from PyQt4.Qsci import QsciScintilla, QsciAPIs, QsciLexerMakefile
 
 from hgext import mq
 
@@ -29,7 +26,8 @@ class QFoldDialog(QDialog):
         self.setWindowIcon(qtlib.geticon('hg-qfold'))
 
         f = self.windowFlags()
-        self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint)
+        self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint
+                            | Qt.WindowMaximizeButtonHint)
 
         self.setLayout(QVBoxLayout())
 
@@ -108,7 +106,6 @@ class QFoldDialog(QDialog):
 
         self.msgte.setText(self.composeMsg(self.patches))
         self.msgte.refresh(self.repo)
-        self.focus = self.msgte
 
     def showSummary(self, item):
         patchname = hglib.fromunicode(item.text())
@@ -120,6 +117,7 @@ class QFoldDialog(QDialog):
               [hglib.tounicode(self.repo.changectx(p).description())
                for p in ['qtip'] + patches])
 
+    @pyqtSlot()
     def configChanged(self):
         '''Repository is reporting its config files have changed'''
         self.msgte.refresh(self.repo)
