@@ -9,8 +9,8 @@ import os
 
 from hgext import mq
 
-from tortoisehg.hgqt import qtlib, thgrepo, qrename
-from tortoisehg.util import hglib, paths
+from tortoisehg.hgqt import qtlib, qrename
+from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
 
 from PyQt4.QtCore import *
@@ -71,8 +71,7 @@ class QReorderDialog(QDialog):
         self.ulw = PatchListWidget(self)
         self.ulw.setDragDropMode(QListView.InternalMove)
         ugb.layout().addWidget(self.ulw)
-        self.ulw.currentItemChanged.connect(lambda:
-                self.showSummary(self.ulw.item(self.ulw.currentRow())))
+        self.ulw.currentItemChanged.connect(self.showSummary)
         self.ulw.menuRequested.connect(self.patchlistMenuRequest)
         layout.addWidget(ugb)
 
@@ -81,8 +80,7 @@ class QReorderDialog(QDialog):
         agb.layout().setContentsMargins(*(0,)*4)
         self.alw = PatchListWidget(self)
         agb.layout().addWidget(self.alw)
-        self.alw.currentItemChanged.connect(lambda:
-                self.showSummary(self.alw.item(self.alw.currentRow())))
+        self.alw.currentItemChanged.connect(self.showSummary)
         self.alw.menuRequested.connect(self.patchlistMenuRequest)
         layout.addWidget(agb)
 
@@ -116,10 +114,9 @@ class QReorderDialog(QDialog):
     def patchlistMenuRequest(self, point, selection):
         self.menuselection = selection
         menu = QMenu(self)
-        act = QAction(_('Rename patch'), self)
-        act.triggered.connect(self.qrenamePatch)
-        menu.addAction(act)
+        menu.addAction(_('Rename patch'), self.qrenamePatch)
         menu.exec_(point)
+        menu.setParent(None)
 
     def qrenamePatch(self):
         patchname = self.menuselection
@@ -130,6 +127,7 @@ class QReorderDialog(QDialog):
             dlg.makeLogVisible.connect(self.parent().makeLogVisible)
         dlg.exec_()
 
+    @pyqtSlot()
     def refresh(self):
         patchnames = self.repo.mq.series[:]
         applied = [p.name for p in self.repo.mq.applied]
@@ -175,6 +173,7 @@ class QReorderDialog(QDialog):
             alw.setCurrentRow(ar)
             self.alw.setFocus()
 
+    #@pyqtSlot(QListWidgetItem)
     def showSummary(self, item):
         if item is None:
             self.summ.clear()
@@ -184,29 +183,15 @@ class QReorderDialog(QDialog):
 
     def accept(self):
         self._writesettings()
-        try:
-            self.repo.incrementBusyCount()
-            lines = []
-            for i in xrange(self.alw.count()-1, -1, -1):
-                item = self.alw.item(i)
-                lines.append(item.patchname)
-            for i in xrange(self.ulw.count()-1, -1, -1):
-                item = self.ulw.item(i)
-                lines.append(item.patchname)
-            if lines:
-                fp = self.repo.mq.opener('series', 'wb')
-                fp.write('\n'.join(lines))
-                fp.close()
-        finally:
-            self.repo.decrementBusyCount()
+        applied = [self.alw.item(x).patchname
+                   for x in xrange(self.alw.count())]
+        unapplied = [self.ulw.item(x).patchname
+                     for x in xrange(self.ulw.count())]
+        writeSeries(self.repo, applied, unapplied)
         QDialog.accept(self)
 
-    def reject(self):
-        QDialog.reject(self)
-
     def closeEvent(self, event):
         self._writesettings()
-        self.repo.repositoryChanged.disconnect(self.refresh)
         super(QReorderDialog, self).closeEvent(event)
 
     def _readsettings(self):
@@ -217,10 +202,15 @@ class QReorderDialog(QDialog):
         s = QSettings()
         s.setValue('qreorder/geom', self.saveGeometry())
 
-def run(ui, *pats, **opts):
-    repo = thgrepo.repository(None, paths.find_root())
-    if hasattr(repo, 'mq'):
-        return QReorderDialog(repo)
-    else:
-        qtlib.ErrorMsgBox(_('TortoiseHg Error'),
-            _('Please enable the MQ extension first.'))
+def writeSeries(repo, applied, unapplied):
+    try:
+        repo.incrementBusyCount()
+        # The series file stores the oldest unapplied patch at the bottom, and
+        # the first applied patch at the top
+        lines = reversed(unapplied + applied)
+        if lines:
+            fp = repo.mq.opener('series', 'wb')
+            fp.write('\n'.join(lines))
+            fp.close()
+    finally:
+        repo.decrementBusyCount()
