@@ -9,6 +9,7 @@ import os
 import sys
 import atexit
 import shutil
+import shlex
 import stat
 import subprocess
 import tempfile
@@ -26,7 +27,7 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
 if PYQT_VERSION < 0x40600 or QT_VERSION < 0x40600:
-    sys.stderr.write('TortoiseHg requires Qt 4.6 and PyQt 4.6\n')
+    sys.stderr.write('TortoiseHg requires at least Qt 4.6 and PyQt 4.6\n')
     sys.stderr.write('You have Qt %s and PyQt %s\n' %
                      (QT_VERSION_STR, PYQT_VERSION_STR))
     sys.exit()
@@ -204,13 +205,17 @@ def savefiles(repo, files, rev, parent=None):
     for curfile in files:
         wfile = util.localpath(curfile)
         wfile, ext = os.path.splitext(os.path.basename(wfile))
+        extfilter = [_("All files (*)")]
         if wfile:
             filename = "%s@%d%s" % (wfile, rev, ext)
+            if ext:
+                extfilter.insert(0, "*%s" % ext)
         else:
             filename = "%s@%d" % (ext, rev)
-        result = QFileDialog.getSaveFileName(
-            parent=parent, caption=_("Save file to"),
-            directory=hglib.tounicode(filename))
+
+        result = QFileDialog.getSaveFileName(parent, _("Save file to"),
+                                             hglib.tounicode(filename),
+                                             ";;".join(extfilter))
         if not result:
             continue
         cwd = os.getcwd()
@@ -238,8 +243,15 @@ def openshell(root, reponame, ui=None):
             if args:
                 shell = shell + ' ' + util.expandpath(args)
             shellcmd = shell % {'root': root, 'reponame': reponame}
+
+            # Unix: QProcess.startDetached(program) cannot parse single-quoted
+            # parameters built using util.shellquote().
+            # Windows: subprocess.Popen(program, shell=True) cannot spawn
+            # cmd.exe in new window, probably because the initial cmd.exe is
+            # invoked with SW_HIDE.
             os.chdir(root)
-            started = QProcess.startDetached(shellcmd)
+            fullargs = shlex.split(shellcmd)
+            started = QProcess.startDetached(fullargs[0], fullargs[1:])
         finally:
             os.chdir(cwd)
         if not started:
@@ -249,6 +261,13 @@ def openshell(root, reponame, ui=None):
         InfoMsgBox(_('No shell configured'),
                    _('A terminal shell must be configured'))
 
+
+def isdarktheme(palette=None):
+    """True if white-on-black color scheme is preferable"""
+    if not palette:
+        palette = QApplication.palette()
+    return palette.color(QPalette.Base).black() >= 0x80
+
 # _styles maps from ui labels to effects
 # _effects maps an effect to font style properties.  We define a limited
 # set of _effects, since we convert color effect names to font style
@@ -598,7 +617,7 @@ _fontcache = {}
 def initfontcache(ui):
     for name in _fontdefaults:
         fname = ui.config('tortoisehg', name, _fontdefaults[name])
-        _fontcache[name] = ThgFont(fname)
+        _fontcache[name] = ThgFont(hglib.tounicode(fname))
 
 def getfont(name):
     assert name in _fontdefaults
@@ -1144,6 +1163,161 @@ class WidgetGroups(object):
     def set_enable(self, *args, **kargs):
         self.set_prop('setEnabled', *args, **kargs)
 
+class DialogKeeper(QObject):
+    """Manage non-blocking dialogs identified by creation parameters
+
+    Example "open single dialog per type":
+
+    >>> mainwin = QWidget()
+    >>> dialogs = DialogKeeper(lambda self, cls: cls(self), parent=mainwin)
+    >>> dlg1 = dialogs.open(QDialog)
+    >>> dlg1.parent() is mainwin
+    True
+    >>> dlg2 = dialogs.open(QDialog)
+    >>> dlg1 is dlg2
+    True
+    >>> dialogs.count()
+    1
+
+    closed dialog will be disowned:
+
+    >>> dlg1.reject()
+    >>> dlg1.parent() is None
+    True
+    >>> dialogs.count()
+    0
+
+    and recreates as necessary:
+
+    >>> dlg3 = dialogs.open(QDialog)
+    >>> dlg1 is dlg3
+    False
+
+    creates new dialog of the same type:
+
+    >>> dlg4 = dialogs.openNew(QDialog)
+    >>> dlg3 is dlg4
+    False
+    >>> dialogs.count()
+    2
+
+    and the last dialog is preferred:
+
+    >>> dialogs.open(QDialog) is dlg4
+    True
+    >>> dlg4.reject()
+    >>> dialogs.count()
+    1
+    >>> dialogs.open(QDialog) is dlg3
+    True
+
+    The following example is not recommended because it creates reference
+    cycles and makes hard to garbage-collect::
+
+        self._dialogs = DialogKeeper(self._createDialog)
+        self._dialogs = DialogKeeper(lambda *args: Foo(self))
+
+    When to delete reference:
+
+    If a dialog is not referenced, and if accept(), reject() and done() are
+    not overridden, it could be garbage-collected during finished signal and
+    lead to hard crash::
+
+        #0  isSignalConnected (signal_index=..., this=0x0)
+        #1  QMetaObject::activate (...)
+        #2  ... in sipQDialog::done (this=0x1d655b0, ...)
+        #3  ... in sipQDialog::reject (this=0x1d655b0)
+        #4  ... in QDialog::closeEvent (this=this@entry=0x1d655b0, ...)
+
+    To avoid crash, a finished dialog is referenced explicitly by DialogKeeper
+    until next event processing.
+
+    >>> dialogs = DialogKeeper(QDialog)
+    >>> dlgref = weakref.ref(dialogs.open())
+    >>> dialogs.count()
+    1
+    >>> esckeyev = QKeyEvent(QEvent.KeyPress, Qt.Key_Escape, Qt.NoModifier)
+    >>> QApplication.postEvent(dlgref(), esckeyev)  # close without incref
+    >>> QApplication.processEvents()  # should not crash
+    >>> dialogs.count()
+    0
+    >>> dlgref() is None  # nobody should have reference
+    True
+    """
+
+    def __init__(self, createdlg, genkey=None, parent=None):
+        super(DialogKeeper, self).__init__(parent)
+        self._createdlg = createdlg
+        self._genkey = genkey or DialogKeeper._defaultgenkey
+        self._keytodlgs = {}  # key: [dlg, ...]
+        self._dlgtokey = {}   # dlg: key
+
+        self._garbagedlgs = []
+        self._emptygarbagelater = QTimer(self, interval=0, singleShot=True)
+        self._emptygarbagelater.timeout.connect(self._emptygarbage)
+
+    def open(self, *args, **kwargs):
+        """Create new dialog or reactivate existing dialog"""
+        dlg = self._preparedlg(self._genkey(self.parent(), *args, **kwargs),
+                               args, kwargs)
+        dlg.show()
+        dlg.raise_()
+        dlg.activateWindow()
+        return dlg
+
+    def openNew(self, *args, **kwargs):
+        """Create new dialog even if there exists the specified one"""
+        dlg = self._populatedlg(self._genkey(self.parent(), *args, **kwargs),
+                                args, kwargs)
+        dlg.show()
+        dlg.raise_()
+        dlg.activateWindow()
+        return dlg
+
+    def _preparedlg(self, key, args, kwargs):
+        if key in self._keytodlgs:
+            assert len(self._keytodlgs[key]) > 0
+            return self._keytodlgs[key][-1]  # prefer latest
+        else:
+            return self._populatedlg(key, args, kwargs)
+
+    def _populatedlg(self, key, args, kwargs):
+        dlg = self._createdlg(self.parent(), *args, **kwargs)
+        if key not in self._keytodlgs:
+            self._keytodlgs[key] = []
+        self._keytodlgs[key].append(dlg)
+        self._dlgtokey[dlg] = key
+        dlg.finished.connect(self._forgetdlg)
+        return dlg
+
+    #@pyqtSlot()
+    def _forgetdlg(self):
+        dlg = self.sender()
+        dlg.finished.disconnect(self._forgetdlg)
+        if dlg.parent() is self.parent():
+            dlg.setParent(None)  # assist gc
+        key = self._dlgtokey.pop(dlg)
+        self._keytodlgs[key].remove(dlg)
+        if not self._keytodlgs[key]:
+            del self._keytodlgs[key]
+
+        # avoid deletion inside finished signal
+        self._garbagedlgs.append(dlg)
+        self._emptygarbagelater.start()
+
+    @pyqtSlot()
+    def _emptygarbage(self):
+        del self._garbagedlgs[:]
+
+    def count(self):
+        assert len(self._dlgtokey) == sum(len(dlgs) for dlgs
+                                          in self._keytodlgs.itervalues())
+        return len(self._dlgtokey)
+
+    @staticmethod
+    def _defaultgenkey(_parent, *args, **_kwargs):
+        return args
+
 class TaskWidget(object):
     def canswitch(self):
         """Return True if the widget allows to switch away from it"""
@@ -1311,12 +1485,9 @@ class PaletteSwitcher(object):
     def __init__(self, targetwidget):
         self._targetwref = weakref.ref(targetwidget)  # avoid circular ref
         self._defaultpalette = targetwidget.palette()
-        bgcolor = self._defaultpalette.color(QPalette.Base)
-        if bgcolor.black() <= 128:
-            # Light theme
+        if not isdarktheme(self._defaultpalette):
             filterbgcolor = QColor('#FFFFB7')
         else:
-            # Dark theme
             filterbgcolor = QColor('darkgrey')
         self._filterpalette = QPalette()
         self._filterpalette.setColor(QPalette.Base, filterbgcolor)
