@@ -62,6 +62,8 @@ COLUMNHEADERS = (
 UNAPPLIED_PATCH_COLOR = '#999999'
 HIDDENREV_COLOR = '#666666'
 
+GraphRole = Qt.UserRole + 0
+
 def get_color(n, ignore=()):
     """
     Return a color at index 'n' rotating in the available
@@ -73,13 +75,13 @@ def get_color(n, ignore=()):
         colors = COLORS
     return colors[n % len(colors)]
 
-def get_style(line_type):
+def get_style(line_type, active):
     if line_type == LINE_TYPE_GRAFT:
         return Qt.DashLine
     return Qt.SolidLine
 
-def get_width(line_type):
-    if line_type == LINE_TYPE_GRAFT:
+def get_width(line_type, active):
+    if line_type == LINE_TYPE_GRAFT or not active:
         return 1
     return 2
 
@@ -223,6 +225,10 @@ class HgRepoListModel(QAbstractTableModel):
         self.showMessage.emit('')
         QTimer.singleShot(0, self, SIGNAL('filled()'))
 
+    def setRevset(self, revset):
+        self.revset = revset
+        self.invalidateCache()
+
     def reloadConfig(self):
         _ui = self.repo.ui
         self.fill_step = int(_ui.config('tortoisehg', 'graphlimit', 500))
@@ -384,6 +390,7 @@ class HgRepoListModel(QAbstractTableModel):
         return QVariant(pix)
 
     def _drawgraphctx(self, painter, pix, ctx, gnode):
+        revset = self.revset
         h = pix.height()
         dot_y = h / 2
 
@@ -396,17 +403,26 @@ class HgRepoListModel(QAbstractTableModel):
         lpen = QPen(pen)
         lpen.setColor(Qt.black)
         painter.setPen(lpen)
+        if revset:
+            def isactive(start, end, color, line_type, children, rev):
+                return rev in revset and util.any(r in revset for r in children)
+        else:
+            def isactive(start, end, color, line_type, children, rev):
+                return True
+
         for y1, y4, lines in ((dot_y, dot_y + h, gnode.bottomlines),
                               (dot_y - h, dot_y, gnode.toplines)):
             y2 = y1 + 1 * (y4 - y1)/4
             ymid = (y1 + y4)/2
             y3 = y1 + 3 * (y4 - y1)/4
 
-            for start, end, color, line_type in lines:
+            lines = sorted((isactive(*l), l) for l in lines)
+
+            for active, (start, end, color, line_type, children, rev) in lines:
                 lpen = QPen(pen)
-                lpen.setColor(QColor(get_color(color)))
-                lpen.setStyle(get_style(line_type))
-                lpen.setWidth(get_width(line_type))
+                lpen.setColor(QColor(active and get_color(color) or "gray"))
+                lpen.setStyle(get_style(line_type, active))
+                lpen.setWidth(get_width(line_type, active))
                 painter.setPen(lpen)
                 x1 = self.col2x(start)
                 x2 = self.col2x(end)
@@ -421,7 +437,12 @@ class HgRepoListModel(QAbstractTableModel):
                 painter.drawPath(path)
 
         # Draw node
-        dot_color = QColor(self.namedbranch_color(ctx.branch()))
+        if revset and gnode.rev not in revset:
+            dot_color = QColor("gray")
+            radius = self.dotradius * 0.8
+        else:
+            dot_color = QColor(self.namedbranch_color(ctx.branch()))
+            radius = self.dotradius
         dotcolor = dot_color.lighter()
         pencolor = dot_color.darker()
         truewhite = QColor("white")
@@ -432,7 +453,6 @@ class HgRepoListModel(QAbstractTableModel):
         pen.setWidthF(1.5)
         painter.setPen(pen)
 
-        radius = self.dotradius
         centre_x = self.col2x(gnode.x)
         centre_y = h/2
 
@@ -502,7 +522,7 @@ class HgRepoListModel(QAbstractTableModel):
     def _roleoffsets(self):
         return {Qt.DisplayRole : 0,
                 Qt.ForegroundRole : len(self._columns),
-                Qt.DecorationRole : len(self._columns) * 2}
+                GraphRole : len(self._columns) * 2}
 
     def data(self, index, role):
         if not index.isValid():
@@ -534,10 +554,10 @@ class HgRepoListModel(QAbstractTableModel):
             self._cache.extend([None,] * (graphlen-cachelen))
         data = self._cache[row]
         if data is None:
-            data = [None,] * (self._roleoffsets[Qt.DecorationRole]+1)
+            data = [None,] * (self._roleoffsets[GraphRole]+1)
         column = self._columns[index.column()]
         offset = self._roleoffsets[role]
-        if role == Qt.DecorationRole:
+        if role == GraphRole:
             if column != 'Graph':
                 return nullvariant
             if data[offset] is None:
