@@ -9,7 +9,7 @@
 
 from mercurial import error
 
-from tortoisehg.util import hglib
+from tortoisehg.util import hglib, obsoleteutil
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import csinfo, qtlib
 
@@ -26,6 +26,10 @@ def label_func(widget, item, ctx):
         return _('Child:')
     elif item == 'patch':
         return _('Patch:')
+    elif item == 'precursors':
+        return _('Precursors:')
+    elif item == 'successors':
+        return _('Successors:')
     raise csinfo.UnknownItem(item)
 
 def revid_markup(revid, **kargs):
@@ -35,13 +39,16 @@ def revid_markup(revid, **kargs):
 
 def data_func(widget, item, ctx):
     def summary_line(desc):
-        desc = desc.replace('\0', '').split('\n')[0]
-        return hglib.tounicode(desc)[:80]
+        return hglib.longsummary(desc.replace('\0', ''))
     def revline_data(ctx, hl=False, branch=None):
         if isinstance(ctx, basestring):
             return ctx
         desc = ctx.description()
         return (str(ctx.rev()), str(ctx), summary_line(desc), hl, branch)
+    def format_ctxlist(ctxlist):
+        if not ctxlist:
+            return None
+        return [revline_data(ctx)[:3] for ctx in ctxlist]
     if item == 'cset':
         return revline_data(ctx)
     elif item == 'branch':
@@ -68,7 +75,7 @@ def data_func(widget, item, ctx):
                 branch = cctx.branch()
             children.append(revline_data(cctx, branch=branch))
         return children
-    elif item in ('transplant', 'p4', 'svn', 'converted'):
+    elif item in ('graft', 'transplant', 'p4', 'svn', 'converted'):
         ts = widget.get_data(item, usepreset=True)
         if not ts:
             return None
@@ -91,6 +98,13 @@ def data_func(widget, item, ctx):
         if ctx.rev() is None:
             ctx = ctx.p1()
         return ctx.extra().get('close') is not None
+    elif item == 'precursors':
+        ctxlist = obsoleteutil.first_known_precursors(ctx)
+        return format_ctxlist(ctxlist)
+    elif item == 'successors':
+        ctxlist = obsoleteutil.first_known_successors(ctx)
+        return format_ctxlist(ctxlist)
+
     raise csinfo.UnknownItem(item)
 
 def markup_func(widget, item, value):
@@ -118,12 +132,12 @@ def markup_func(widget, item, value):
             if branch:
                 return '%s - %s %s' % (revnum, branch, summary)
             return '%s - %s' % (revnum, summary)
-    if item in ('cset', 'transplant', 'patch', 'p4', 'svn', 'converted'):
+    if item in ('cset', 'graft', 'transplant', 'patch', 'p4', 'svn', 'converted'):
         link = item != 'cset'
         if isinstance(value, basestring):
             return revid_markup(value)
         return revline_markup(link=link, *value)
-    elif item in ('parents', 'children'):
+    elif item in ('parents', 'children', 'precursors', 'successors'):
         csets = []
         for cset in value:
             if isinstance(cset, basestring):
@@ -137,8 +151,9 @@ def RevPanelWidget(repo):
     '''creates a rev panel widget and returns it'''
     custom = csinfo.custom(data=data_func, label=label_func,
                            markup=markup_func)
-    style = csinfo.panelstyle(contents=('cset', 'branch', 'close', 'user',
-                   'dateage', 'parents', 'children', 'tags', 'transplant',
+    style = csinfo.panelstyle(contents=('cset', 'branch', 'obsolete', 'close', 'user',
+                   'dateage', 'parents', 'children', 'tags', 'graft', 'transplant',
+                   'precursors', 'successors',
                    'p4', 'svn', 'converted'), selectable=True,
                    expandable=True)
     return csinfo.create(repo, style=style, custom=custom)
