@@ -8,7 +8,7 @@
 import sys, os, httplib, socket, tempfile
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
-from mercurial import extensions, hgweb, util, error, subrepo
+from mercurial import extensions, hgweb, util, error
 from mercurial.hgweb import server  # workaround for demandimport
 from tortoisehg.util import paths, wconfig, hglib
 from tortoisehg.hgqt import cmdui, qtlib, thgrepo
@@ -61,15 +61,14 @@ class ServeDialog(QDialog):
         self._webconf_form.setEnabled(not self.isstarted())
 
     def _updatestatus(self):
-        def statustext():
-            if self.isstarted():
-                # TODO: escape special chars
-                link = '<a href="%s">%s</a>' % (self.rooturl, self.rooturl)
-                return _('Running at %s') % link
-            else:
-                return _('Stopped')
+        if self.isstarted():
+            # TODO: escape special chars
+            link = '<a href="%s">%s</a>' % (self.rooturl, self.rooturl)
+            msg = _('Running at %s') % link
+        else:
+            msg = _('Stopped')
 
-        self._qui.status_edit.setText(statustext())
+        self._qui.status_edit.setText(msg)
 
     @pyqtSlot()
     def start(self):
@@ -193,6 +192,8 @@ class ServeDialog(QDialog):
     @pyqtSlot()
     def _minimizetotray(self):
         self._trayicon.show()
+        self._trayicon.showMessage(_('TortoiseHg Web Server'),
+                                   _('Running at %s') % self.rooturl)
         self.hide()
 
     @pyqtSlot()
@@ -215,12 +216,13 @@ def _create_server(orig, ui, app):
     def serve_forever(orig):
         server._serving = True
         try:
-            while server._serving:
-                server.handle_request()
-        except KeyboardInterrupt:
-            # raised outside try-block around process_request().
-            # see SocketServer.BaseServer
-            pass
+            try:
+                while server._serving:
+                    server.handle_request()
+            except KeyboardInterrupt:
+                # raised outside try-block around process_request().
+                # see SocketServer.BaseServer
+                pass
         finally:
             server._serving = False
             server.server_close()
@@ -264,17 +266,22 @@ def run(ui, *pats, **opts):
         dlg.start()
     return dlg
 
-def recursiveRepoSearch(repo):
-    yield repo.root
-    try:
-        wctx = repo[None]
-        for s in wctx.substate:
-            sub = wctx.sub(s)
-            if isinstance(sub, subrepo.hgsubrepo):
-                for root in recursiveRepoSearch(sub._repo):
-                    yield root
-    except (EnvironmentError, error.Abort, error.RepoError):
-        pass
+def _asconfigliststr(value):
+    r"""
+    >>> _asconfigliststr('foo')
+    'foo'
+    >>> _asconfigliststr('foo bar')
+    '"foo bar"'
+    >>> _asconfigliststr('foo,bar')
+    '"foo,bar"'
+    >>> _asconfigliststr('foo "bar"')
+    '"foo \\"bar\\""'
+    """
+    # ui.configlist() uses isspace(), which is locale-dependent
+    if util.any(c.isspace() or c == ',' for c in value):
+        return '"' + value.replace('"', '\\"') + '"'
+    else:
+        return value
 
 def _newwebconf(repopath, webconfpath):
     """create config obj for hgweb"""
@@ -286,15 +293,17 @@ def _newwebconf(repopath, webconfpath):
     elif repopath:  # imitate webconf for single repo
         c = wconfig.config()
         try:
+            # TODO: not nice to instantiate repo just for repo.shortname
             repo = thgrepo.repository(None, repopath)
-            roots = [root for root in recursiveRepoSearch(repo)]
-            if len(roots) == 1:
+            if not os.path.exists(os.path.join(repopath, '.hgsub')):
+                # no _asconfigliststr(repopath) for now, because ServeDialog
+                # cannot parse it as a list in single-repo mode.
                 c.set('paths', '/', repopath)
             else:
+                # since hg 8cbb59124e67, path entry is parsed as a list
                 base = hglib.fromunicode(repo.shortname)
-                c.set('paths', base, repopath)
-                for root in roots[1:]:
-                    c.set('paths', base + root[len(repopath):], root)
+                c.set('paths', base,
+                      _asconfigliststr(os.path.join(repopath, '**')))
         except (EnvironmentError, error.Abort, error.RepoError):
             c.set('paths', '/', repopath)
         return c
