@@ -6,12 +6,12 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import os
 import Queue
 import time
 import urllib2, urllib
 import socket
 import errno
+import inspect
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -23,13 +23,6 @@ from tortoisehg.util import thread2, hglib
 from tortoisehg.hgqt.i18n import _, localgettext
 from tortoisehg.hgqt import qtlib
 
-try:
-    _SubrepoAbort = subrepo.SubrepoAbort
-except AttributeError:
-    # hg <= 2.4
-    class _SubrepoAbort(Exception):
-        pass
-
 local = localgettext()
 
 class DataWrapper(object):
@@ -125,12 +118,21 @@ class QtUi(uimod.ui):
         if not self.interactive(): return default
         return self.sig.prompt(msg, default)
 
-    def promptchoice(self, msg, choices, default=0):
+    def promptchoice(self, prompt, default=0):
+        parts = prompt.split('$$')
+        msg = parts[0].rstrip(' ')
+        choices = [p.strip(' ') for p in parts[1:]]
+        return self._promptchoice(msg, choices, default)
+
+    def _promptchoice(self, msg, choices, default=0):
         if not self.interactive(): return default
         return self.sig.promptchoice(msg, choices, default)
 
-    def getpass(self, prompt=_('password: '), default=None):
-        return self.sig.getpass(prompt, default)
+    if len(inspect.getargspec(uimod.ui.promptchoice)[0]) > 3:
+        promptchoice = _promptchoice  # hg<2.7
+
+    def getpass(self, prompt=None, default=None):
+        return self.sig.getpass(prompt or _('password: '), default)
 
     def progress(self, topic, pos, item='', unit='', total=None):
         return self.sig.progress(topic, pos, item, unit, total)
@@ -273,9 +275,13 @@ class CmdThread(QThread):
 
             for k, v in ui.configitems('defaults'):
                 ui.setconfig('defaults', k, '')
+            # disable worker because it only works in main thread:
+            #   signal.signal(signal.SIGINT, signal.SIG_IGN)
+            #   ValueError: signal only works in main thread
+            ui.setconfig('worker', 'numcpus', 1)
             self.ret = 255
             self.ret = hglib.dispatch(ui, self.cmdline) or 0
-        except _SubrepoAbort, e:
+        except subrepo.SubrepoAbort, e:
             errormsg = str(e)
             label = 'ui.error'
             if e.subrepo:
@@ -327,6 +333,14 @@ class CmdThread(QThread):
         except error.LockUnavailable, inst:
             ui.warn(local._("abort: could not lock %s: %s\n") %
                 (inst.desc or inst.filename, inst.strerror))
+        except error.CommandError, inst:
+            if inst.args[0]:
+                ui.warn(local._("hg %s: %s\n") % (inst.args[0], inst.args[1]))
+            else:
+                ui.warn(local._("hg: %s\n") % inst.args[1])
+        except error.OutOfBandError, inst:
+            ui.warn(local._("abort: remote error:\n"))
+            ui.warn(''.join(inst.args))
         except error.RepoError, inst:
             ui.warn(local._("abort: %s!\n") % inst)
         except error.ResponseError, inst:
@@ -339,6 +353,11 @@ class CmdThread(QThread):
                 ui.warn("\n%r\n" % util.ellipsis(inst.args[1]))
         except error.RevlogError, inst:
             ui.warn(local._("abort: %s!\n") % inst)
+        except error.UnknownCommand, inst:
+            ui.warn(local._("hg: unknown command '%s'\n") % inst.args[0])
+        except error.InterventionRequired, inst:
+            ui.warn("%s\n" % inst)
+            self.ret = 1
         except socket.error, inst:
             ui.warn(local._("abort: %s!\n") % str(inst))
         except IOError, inst:
@@ -366,10 +385,8 @@ class CmdThread(QThread):
             else:
                 ui.warn(local._("abort: %s\n") % inst.strerror)
         except Exception, inst:
-            if 'THGDEBUG' in os.environ:
-                import traceback
-                traceback.print_exc()
             ui.write_err(str(inst) + '\n')
+            raise
         except KeyboardInterrupt:
             self.ret = -1
 
