@@ -10,22 +10,19 @@ Main Qt4 application for TortoiseHg
 
 import os
 import sys
-from mercurial import ui, util
 from mercurial.error import RepoError
 from tortoisehg.util import paths, hglib
 
-from tortoisehg.hgqt import thgrepo, cmdui, qtlib, mq
+from tortoisehg.hgqt import cmdui, qtlib, mq, serve
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt.repowidget import RepoWidget
 from tortoisehg.hgqt.reporegistry import RepoRegistryView
 from tortoisehg.hgqt.logcolumns import ColumnSelectDialog
 from tortoisehg.hgqt.docklog import LogDockWidget
 from tortoisehg.hgqt.settings import SettingsDialog
-from tortoisehg.hgqt.run import portable_start_fork
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
-from PyQt4.QtNetwork import QLocalServer, QLocalSocket
 
 class ThgTabBar(QTabBar):
     def mouseReleaseEvent(self, event):
@@ -38,15 +35,20 @@ class ThgTabBar(QTabBar):
 class Workbench(QMainWindow):
     """hg repository viewer/browser application"""
     finished = pyqtSignal(int)
-    activeRepoChanged = pyqtSignal(QString)
 
-    def __init__(self, createserver=False):
+    def __init__(self, ui, repomanager):
         QMainWindow.__init__(self)
         self.progressDialog = QProgressDialog(
             'TortoiseHg - Initializing Workbench', QString(), 0, 100)
         self.progressDialog.setAutoClose(False)
 
-        self.ui = ui.ui()
+        self.ui = ui
+        self._repomanager = repomanager
+        self._repomanager.configChanged.connect(self._setupUrlComboIfCurrent)
+        self._repomanager.configChanged.connect(self._updateRepoShortName)
+        self._repomanager.repositoryChanged.connect(self._updateRepoBaseNode)
+        self._repomanager.repositoryDestroyed.connect(self.closeRepo)
+        self._repomanager.repositoryOpened.connect(self._updateRepoRegItem)
 
         self.setupUi()
         self.setWindowTitle(_('TortoiseHg Workbench'))
@@ -56,9 +58,9 @@ class Workbench(QMainWindow):
         rr.openRepo.connect(self.openRepo)
         rr.removeRepo.connect(self.closeRepo)
         rr.progressReceived.connect(self.progress)
+        self._repomanager.repositoryChanged.connect(rr.scanRepo)
         rr.hide()
         self.addDockWidget(Qt.LeftDockWidgetArea, rr)
-        self.activeRepoChanged.connect(rr.setActiveTabRepo)
 
         self.mqpatches = p = mq.MQPatchesWidget(self)
         p.setObjectName('MQPatchesWidget')
@@ -66,7 +68,7 @@ class Workbench(QMainWindow):
         p.hide()
         self.addDockWidget(Qt.LeftDockWidgetArea, p)
 
-        self.log = LogDockWidget(self)
+        self.log = LogDockWidget(repomanager, self)
         self.log.setObjectName('Log')
         self.log.progressReceived.connect(self.statusbar.progress)
         self.log.hide()
@@ -99,18 +101,8 @@ class Workbench(QMainWindow):
         self.lastClosedRepoRootList = []
         self.progressDialog.close()
         self.progressDialog = None
-        self._dialogs = []
-
-        self.server = None
-        if createserver:
-            # Enable the Workbench Server that is used to maintain a single
-            # workbench instance
-            self.createWorkbenchServer()
-
-    def _forgetdialog(self, dlg):
-        """forget the dialog to be garbage collectable"""
-        assert dlg in self._dialogs
-        self._dialogs.remove(dlg)
+        self._dialogs = qtlib.DialogKeeper(
+            lambda self, dlgmeth: dlgmeth(self), parent=self)
 
     def setupUi(self):
         desktopgeom = qApp.desktop().availableGeometry()
@@ -249,8 +241,8 @@ class Workbench(QMainWindow):
         tasklist = self.ui.configlist(
             'tortoisehg', 'workbench.task-toolbar', [])
         if tasklist == []:
-            tasklist = ['log', 'commit', 'mq', 'sync', 'manifest',
-                'grep', 'pbranch']
+            tasklist = ['log', 'commit', 'mq', 'manifest',
+                'grep', 'pbranch', '|', 'sync']
 
         self.actionSelectTaskMQ = None
         self.actionSelectTaskPbranch = None
@@ -286,8 +278,9 @@ class Workbench(QMainWindow):
                   enabled='repoopen', menu='view', shortcut='Ctrl+/',
                   tooltip=_('Go to a specific revision'))
 
-        newaction(_("Start &Web Server"), self.serve, enabled='repoopen',
-                  menu='repository')
+        menuSync = self.menuRepository.addMenu(_('S&ynchronize'))
+        newseparator(menu='repository')
+        newaction(_("Start &Web Server"), self.serve, menu='repository')
         newseparator(menu='repository')
         newaction(_("&Shelve..."), self._repofwd('shelve'), icon='shelve',
                   enabled='repoopen', menu='repository')
@@ -335,11 +328,11 @@ class Workbench(QMainWindow):
         self.actionCurrentRev = \
         newaction(_("Go to current revision"), self._repofwd('gotoParent'),
                   icon='go-home', tooltip=_('Go to current revision'),
-                  enabled=True, toolbar='edit', shortcut='Ctrl+.')
+                  enabled='repoopen', toolbar='edit', shortcut='Ctrl+.')
         self.actionGoTo = \
         newaction(_("Go to a specific revision"), self.gotorev,
                   icon='go-to-rev', tooltip=_('Go to a specific revision'),
-                  enabled=True, toolbar='edit')
+                  enabled='repoopen', toolbar='edit')
         self.actionBack = \
         newaction(_("Back"), self._repofwd('back'), icon='go-previous',
                   enabled=False, toolbar='edit')
@@ -362,18 +355,21 @@ class Workbench(QMainWindow):
         menu.addAction(self.customtbar.toggleViewAction())
         self.menuView.addMenu(menu)
 
-        newaction(_('Incoming'), data='incoming', icon='hg-incoming',
+        newaction(_('&Incoming'), data='incoming', icon='hg-incoming',
+                  enabled='repoopen', toolbar='sync', shortcut='Ctrl+Shift+,')
+        newaction(_('&Pull'), data='pull', icon='hg-pull',
                   enabled='repoopen', toolbar='sync')
-        newaction(_('Pull'), data='pull', icon='hg-pull',
-                  enabled='repoopen', toolbar='sync')
-        newaction(_('Outgoing'), data='outgoing', icon='hg-outgoing',
-                  enabled='repoopen', toolbar='sync')
-        newaction(_('Push'), data='push', icon='hg-push',
+        newaction(_('&Outgoing'), data='outgoing', icon='hg-outgoing',
+                  enabled='repoopen', toolbar='sync', shortcut='Ctrl+Shift+.')
+        newaction(_('P&ush'), data='push', icon='hg-push',
                   enabled='repoopen', toolbar='sync')
+        menuSync.addActions(self.synctbar.actions())
         self.urlCombo = QComboBox(self)
         self.urlCombo.setSizeAdjustPolicy(QComboBox.AdjustToContents)
         self.urlCombo.currentIndexChanged.connect(self._updateSyncUrlToolTip)
-        self.synctbar.addWidget(self.urlCombo)
+        self.urlComboAction = self.synctbar.addWidget(self.urlCombo)
+        # hide it because workbench could be started without open repo
+        self.urlComboAction.setVisible(False)
         self.synctbar.actionTriggered.connect(self._runSyncAction)
 
         self.updateMenu()
@@ -384,6 +380,11 @@ class Workbench(QMainWindow):
                          for alias, path in repo.ui.configitems('paths'))
         aliases = pathdict.keys()
 
+        combo_setting = repo.ui.config('tortoisehg', 'workbench.target-combo',
+                                       'auto')
+        self.urlComboAction.setVisible(len(aliases) > 1
+                                       or combo_setting == 'always')
+
         # 1. Sort the list if aliases
         aliases.sort()
         # 2. Place the default alias at the top of the list
@@ -431,10 +432,10 @@ class Workbench(QMainWindow):
         self.urlCombo.blockSignals(False)
         self._updateSyncUrlToolTip(self.urlCombo.currentIndex())
 
-    #@pyqtSlot()
-    def _setupUrlComboIfCurrent(self):
+    @pyqtSlot(unicode)
+    def _setupUrlComboIfCurrent(self, root):
         w = self.repoTabsWidget.currentWidget()
-        if self.sender() is w:
+        if w.repoRootPath() == root:
             self._setupUrlCombo(w.repo)
 
     def _syncUrlFor(self, op):
@@ -593,6 +594,15 @@ class Workbench(QMainWindow):
                 act.triggered.connect(cb)
             self.addAction(act)
 
+    def createPopupMenu(self):
+        """Create new popup menu for toolbars and dock widgets"""
+        menu = super(Workbench, self).createPopupMenu()
+        assert menu  # should have toolbar/dock menu
+        menu.addSeparator()
+        menu.addAction(_('Custom Toolbar &Settings'),
+                       self._editCustomToolsSettings)
+        return menu
+
     @pyqtSlot(QPoint)
     def tabBarContextMenuRequest(self, point):
         # Activate the clicked tab
@@ -632,6 +642,7 @@ class Workbench(QMainWindow):
         if actionlist:
             contextmenu.exec_(self.repoTabsWidget.mapToGlobal(point))
 
+    @pyqtSlot()
     def closeLastClickedTab(self):
         if self.repoTabsWidget.lastClickedTab > -1:
             self.repoTabCloseRequested(self.repoTabsWidget.lastClickedTab)
@@ -649,6 +660,7 @@ class Workbench(QMainWindow):
             self.lastClosedRepoRootList = closedRepoRootList
 
 
+    @pyqtSlot()
     def closeNotLastClickedTabs(self):
         self._closeOtherTabs(self.repoTabsWidget.lastClickedTab)
 
@@ -658,30 +670,37 @@ class Workbench(QMainWindow):
             rw.switchToNamedTaskTab(str(action.data().toString()))
 
     @pyqtSlot(QString, bool)
-    def openRepo(self, root, reuse):
-        """ Open repo by openRepoSignal from reporegistry [unicode] """
-        root = hglib.fromunicode(root)
-        self._openRepo(root, reuse)
+    def openRepo(self, root, reuse, bundle=None):
+        """Open tab of the specified repo [unicode]"""
+        root = unicode(root)
+        if root and not root.startswith('ssh://'):
+            if reuse:
+                for rw in self._findRepoWidget(root):
+                    self.repoTabsWidget.setCurrentWidget(rw)
+                    return
+            try:
+                repoagent = self._repomanager.openRepoAgent(root)
+                self.addRepoTab(repoagent, bundle)
+            except RepoError, e:
+                qtlib.WarningMsgBox(_('Failed to open repository'),
+                                    hglib.tounicode(str(e)), parent=self)
 
     @pyqtSlot(QString)
     def closeRepo(self, root):
-        """ Close tab if the repo is removed from reporegistry [unicode] """
-        root = os.path.normpath(hglib.fromunicode(root))
-        for i in xrange(self.repoTabsWidget.count()):
-            w = self.repoTabsWidget.widget(i)
-            if w.repo.root == root:
-                self.repoTabCloseRequested(i)
-                return
+        """Close tabs of the specified repo [unicode]"""
+        for rw in list(self._findRepoWidget(unicode(root))):
+            self.repoTabCloseRequested(self.repoTabsWidget.indexOf(rw))
 
     @pyqtSlot(QString)
     def openLinkedRepo(self, path):
-        uri = path.split('?')
+        uri = unicode(path).split('?', 1)
         path = uri[0]
         rev = None
         if len(uri) > 1:
             rev = hglib.fromunicode(uri[1])
-        rw = self.showRepo(path)
-        if rw:
+        self.showRepo(path)
+        rw = self.repoTabsWidget.currentWidget()
+        if rw and rw.repoRootPath() == os.path.normpath(path):
             if rev:
                 rw.goto(rev)
             else:
@@ -692,19 +711,13 @@ class Workbench(QMainWindow):
     @pyqtSlot(QString)
     def showRepo(self, root):
         """Activate the repo tab or open it if not available [unicode]"""
-        root = unicode(root)
-        for i in xrange(self.repoTabsWidget.count()):
-            w = self.repoTabsWidget.widget(i)
-            if hglib.tounicode(w.repo.root) == os.path.normpath(root):
-                self.repoTabsWidget.setCurrentIndex(i)
-                return w
-        return self._openRepo(hglib.fromunicode(root), False)
+        self.openRepo(root, True)
 
     @pyqtSlot(unicode, QString)
     def setRevsetFilter(self, path, filter):
         for i in xrange(self.repoTabsWidget.count()):
             w = self.repoTabsWidget.widget(i)
-            if hglib.tounicode(w.repo.root) == path:
+            if w.repoRootPath() == path:
                 w.setFilter(filter)
                 return
 
@@ -804,34 +817,36 @@ class Workbench(QMainWindow):
         self.actionBack.setEnabled(rw.canGoBack())
         self.actionForward.setEnabled(rw.canGoForward())
 
-    def repoTabCloseSelf(self, widget):
-        self.repoTabsWidget.setCurrentWidget(widget)
-        index = self.repoTabsWidget.currentIndex()
-        if widget.closeRepoWidget():
-            w = self.repoTabsWidget.widget(index)
-            try:
-                reporoot = hglib.tounicode(w.repo.root)
-            except:
-                reporoot = ''
-            self.repoTabsWidget.removeTab(index)
-            widget.deleteLater()
-            self.updateMenu()
-            self.lastClosedRepoRootList = [reporoot]
+    # might be better to move them to RepoRegistry
+    @pyqtSlot(unicode)
+    def _updateRepoRegItem(self, root):
+        self._updateRepoShortName(root)
+        self._updateRepoBaseNode(root)
 
+    @pyqtSlot(unicode)
+    def _updateRepoShortName(self, root):
+        repo = self._repomanager.repoAgent(root).rawRepo()
+        self.reporegistry.setShortName(root, repo.shortname)
+
+    @pyqtSlot(unicode)
+    def _updateRepoBaseNode(self, root):
+        repo = self._repomanager.repoAgent(root).rawRepo()
+        self.reporegistry.setBaseNode(root, repo[0].node())
+
+    @pyqtSlot(int)
     def repoTabCloseRequested(self, index):
         tw = self.repoTabsWidget
         if 0 <= index < tw.count():
             w = tw.widget(index)
-            try:
-                reporoot = hglib.tounicode(w.repo.root)
-            except:
-                reporoot = ''
-            if w and w.closeRepoWidget():
+            reporoot = w.repoRootPath()
+            if w.closeRepoWidget():
                 tw.removeTab(index)
                 w.deleteLater()
+                self._repomanager.releaseRepoAgent(reporoot)
                 self.updateMenu()
                 self.lastClosedRepoRootList = [reporoot]
 
+    @pyqtSlot()
     def reopenLastClosedTabs(self):
         for n, reporoot in enumerate(self.lastClosedRepoRootList):
             self.progress(_('Reopening tabs'), n,
@@ -842,20 +857,20 @@ class Workbench(QMainWindow):
         self.lastClosedRepoRootList = []
         self.progress(_('Reopening tabs'), None, '', '', None)
 
-    def repoTabChanged(self, index=0):
+    @pyqtSlot()
+    def repoTabChanged(self):
         w = self.repoTabsWidget.currentWidget()
         if w:
             self.updateHistoryActions()
             self.updateMenu()
-            if w.repo:
-                root = w.repo.root
-                self.activeRepoChanged.emit(hglib.tounicode(root))
-                self._setupCustomTools(w.repo.ui)
-                self._setupUrlCombo(w.repo)
+            self.log.setCurrentRepoRoot(w.repoRootPath())
+            self.reporegistry.setActiveTabRepo(w.repoRootPath())
+            self._setupCustomTools(w.repo.ui)
+            self._setupUrlCombo(w.repo)
         else:
-            self.activeRepoChanged.emit("")
+            self.log.setCurrentRepoRoot(None)
+            self.reporegistry.setActiveTabRepo('')
         repo = w and w.repo or None
-        self.log.setRepository(repo)
         self.mqpatches.setrepo(repo)
 
     #@pyqtSlot(unicode)
@@ -865,24 +880,23 @@ class Workbench(QMainWindow):
         if index == self.repoTabsWidget.currentIndex():
             self._updateWindowTitle()
 
-    def addRepoTab(self, repo, bundle):
+    #@pyqtSlot(QIcon)
+    def _updateRepoTabIcon(self, icon):
+        index = self.repoTabsWidget.indexOf(self.sender())
+        self.repoTabsWidget.setTabIcon(index, icon)
+
+    def addRepoTab(self, repoagent, bundle):
         '''opens the given repo in a new tab'''
-        rw = RepoWidget(repo, self, bundle=bundle)
+        rw = RepoWidget(repoagent, self, bundle=bundle)
         rw.showMessageSignal.connect(self.showMessage)
-        rw.closeSelfSignal.connect(self.repoTabCloseSelf)
         rw.progress.connect(lambda tp, p, i, u, tl:
-            self.statusbar.progress(tp, p, i, u, tl, repo.root))
-        rw.output.connect(self.log.output)
+            self.statusbar.progress(tp, p, i, u, tl, rw.repo.root))
+        rw.output.connect(self._appendRepoWidgetOutput)
         rw.makeLogVisible.connect(self.log.setShown)
-        rw.beginSuppressPrompt.connect(self.log.beginSuppressPrompt)
-        rw.endSuppressPrompt.connect(self.log.endSuppressPrompt)
         rw.revisionSelected.connect(self.updateHistoryActions)
         rw.repoLinkClicked.connect(self.openLinkedRepo)
         rw.taskTabsWidget.currentChanged.connect(self.updateTaskViewMenu)
         rw.toolbarVisibilityChanged.connect(self.updateToolBarActions)
-        rw.shortNameChanged.connect(self.reporegistry.shortNameChanged)
-        rw.baseNodeChanged.connect(self.reporegistry.baseNodeChanged)
-        rw.repoChanged.connect(self.reporegistry.scanRepo)
 
         tw = self.repoTabsWidget
         # We can open new tabs next to the current one or next to the last tab
@@ -893,18 +907,20 @@ class Workbench(QMainWindow):
                 tw.currentIndex()+1, rw, rw.title())
         else:
             index = self.repoTabsWidget.addTab(rw, rw.title())
-        tw.setTabToolTip(index, hglib.tounicode(repo.root))
+        tw.setTabToolTip(index, repoagent.rootPath())
         tw.setCurrentIndex(index)
         rw.titleChanged.connect(self._updateRepoTabTitle)
-        rw.repoConfigChanged.connect(self._setupUrlComboIfCurrent)
-        rw.showIcon.connect(
-            lambda icon: tw.setTabIcon(tw.indexOf(rw), icon))
-        self.reporegistry.addRepo(hglib.tounicode(repo.root))
+        rw.showIcon.connect(self._updateRepoTabIcon)
+        self.reporegistry.addRepo(repoagent.rootPath())
 
         self.updateMenu()
         return rw
 
-
+    #@pyqtSlot(QString, QString)
+    def _appendRepoWidgetOutput(self, msg, label):
+        rw = self.sender()
+        assert isinstance(rw, RepoWidget)
+        self.log.appendLog(msg, label, rw.repoRootPath())
 
     def showMessage(self, msg):
         self.statusbar.showMessage(msg)
@@ -963,7 +979,12 @@ class Workbench(QMainWindow):
             getattr(w, 'reload')()
             self._setupUrlCombo(w.repo)
 
-    @pyqtSlot(QAction)
+        if not self.mqpatches.isHidden():
+            self.mqpatches.reload()
+
+    # no @pyqtSlot(QAction) to avoid wrong self.sender() at
+    # _appendRepoWidgetOutput, _updateRepoTabTitle and _updateRepoTabIcon
+    # during QMessageBox.exec at SyncWidget.pushclicked. See #3320.
     def _runSyncAction(self, action):
         w = self.repoTabsWidget.currentWidget()
         if w:
@@ -972,10 +993,14 @@ class Workbench(QMainWindow):
             getattr(w, op)()
 
     def serve(self):
+        self._dialogs.open(Workbench._createServeDialog)
+
+    def _createServeDialog(self):
         w = self.repoTabsWidget.currentWidget()
         if w:
-            from tortoisehg.hgqt import run
-            run.serve(w.repo.ui, root=w.repo.root)
+            return serve.run(w.repo.ui, root=w.repo.root)
+        else:
+            return serve.run(self.ui)
 
     def loadall(self):
         w = self.repoTabsWidget.currentWidget()
@@ -991,6 +1016,7 @@ class Workbench(QMainWindow):
             w.repoview.goto(rev)
 
     def newWorkbench(self):
+        from tortoisehg.hgqt.run import portable_start_fork
         portable_start_fork(['--new'])
 
     def newRepository(self):
@@ -1005,23 +1031,23 @@ class Workbench(QMainWindow):
         dlg.finished.connect(dlg.deleteLater)
         if dlg.exec_():
             path = dlg.getPath()
-            self._openRepo(path, False)
+            self.openRepo(hglib.tounicode(path), False)
 
     def cloneRepository(self):
         """ Run clone dialog """
-        from tortoisehg.hgqt.clone import CloneDialog
+        # it might be better to reuse existing CloneDialog
+        dlg = self._dialogs.openNew(Workbench._createCloneDialog)
         repoWidget = self.repoTabsWidget.currentWidget()
         if repoWidget:
-            root = repoWidget.repo.root
-            args = [root, root + '-clone']
-        else:
-            args = []
-        dlg = CloneDialog(args, parent=self)
-        dlg.finished.connect(dlg.deleteLater)
+            uroot = repoWidget.repoRootPath()
+            dlg.setSource(uroot)
+            dlg.setDestination(uroot + '-clone')
+
+    def _createCloneDialog(self):
+        from tortoisehg.hgqt.clone import CloneDialog
+        dlg = CloneDialog(parent=self)
         dlg.clonedRepository.connect(self.showRepo)
-        dlg.destroyed.connect(lambda: self._forgetdialog(dlg))
-        dlg.show()
-        self._dialogs.append(dlg)
+        return dlg
 
     def openRepository(self):
         """ Open repo from File menu """
@@ -1035,30 +1061,17 @@ class Workbench(QMainWindow):
         FD = QFileDialog
         path = FD.getExistingDirectory(self, caption, cwd,
                                        FD.ShowDirsOnly | FD.ReadOnly)
-        self._openRepo(hglib.fromunicode(path), False)
+        self.openRepo(path, False)
 
-    def _openRepo(self, root, reuse, bundle=None):
-        if root and not root.startswith('ssh://'):
-            if reuse:
-                for rw in self._findrepowidget(root):
-                    self.repoTabsWidget.setCurrentWidget(rw)
-                    return
-            try:
-                repo = thgrepo.repository(path=root)
-                return self.addRepoTab(repo, bundle)
-            except RepoError, e:
-                qtlib.WarningMsgBox(_('Failed to open repository'),
-                                    hglib.tounicode(str(e)), parent=self)
-        return None
-
-    def _findrepowidget(self, root):
+    def _findRepoWidget(self, root):
         """Iterates RepoWidget for the specified root"""
         def normpathandcase(path):
-            return os.path.normcase(util.normpath(path))
+            return os.path.normcase(os.path.normpath(path))
+        normroot = normpathandcase(root)
         tw = self.repoTabsWidget
         for idx in range(tw.count()):
             rw = tw.widget(idx)
-            if normpathandcase(rw.repo.root) == normpathandcase(root):
+            if normpathandcase(rw.repoRootPath()) == normroot:
                 yield rw
 
     def onAbout(self, *args):
@@ -1159,10 +1172,10 @@ class Workbench(QMainWindow):
             tw = self.repoTabsWidget
             for idx in range(tw.count()):
                 rw = tw.widget(idx)
-                repostosave.append(hglib.tounicode(rw.repo.root))
+                repostosave.append(rw.repoRootPath())
             cw = tw.currentWidget()
             if cw is not None:
-                lastactiverepo = hglib.tounicode(cw.repo.root)
+                lastactiverepo = cw.repoRootPath()
         s.setValue(wb + 'lastactiverepo', lastactiverepo)
         s.setValue(wb + 'openrepos', (',').join(repostosave))
 
@@ -1189,16 +1202,16 @@ class Workbench(QMainWindow):
                           _('Reopening repository %s') % upath, '',
                           len(openrepos))
             QCoreApplication.processEvents()
-            self._openRepo(hglib.fromunicode(upath), False)
+            self.openRepo(upath, False)
             QCoreApplication.processEvents()
         self.progress(_('Reopening tabs'), None, '', '', None)
 
         # Activate the tab that was last active on the last session (if any)
         # Note that if a "root" has been passed to the "thg" command,
         # this will have no effect
-        lastactiverepo = hglib.fromunicode(s.value(wb + 'lastactiverepo').toString())
-        if lastactiverepo != '':
-            self._openRepo(lastactiverepo, True)
+        lastactiverepo = s.value(wb + 'lastactiverepo').toString()
+        if lastactiverepo:
+            self.openRepo(lastactiverepo, True)
 
         # Clear the lastactiverepo and the openrepos list once the workbench state
         # has been reload, so that opening additional workbench windows does not
@@ -1207,7 +1220,7 @@ class Workbench(QMainWindow):
         s.setValue(wb + 'lastactiverepo', '')
 
     def goto(self, root, rev):
-        for rw in self._findrepowidget(root):
+        for rw in self._findRepoWidget(hglib.tounicode(root)):
             rw.goto(rev)
 
     def closeEvent(self, event):
@@ -1216,8 +1229,6 @@ class Workbench(QMainWindow):
         else:
             self.storeSettings()
             self.reporegistry.close()
-            if self.server:
-                self.server.close()
             # mimic QDialog exit
             self.finished.emit(0)
 
@@ -1246,140 +1257,15 @@ class Workbench(QMainWindow):
         if w:
             qtlib.openshell(w.repo.root, w.repo.displayname, w.repo.ui)
 
-    def editSettings(self):
+    @pyqtSlot()
+    def editSettings(self, focus=None):
         tw = self.repoTabsWidget
         w = tw.currentWidget()
         twrepo = (w and w.repo.root or '')
-        sd = SettingsDialog(configrepo=False,
+        sd = SettingsDialog(configrepo=False, focus=focus,
                             parent=self, root=twrepo)
         sd.exec_()
 
-    def createWorkbenchServer(self):
-        self.server = QLocalServer()
-        self.server.newConnection.connect(self.newConnection)
-        self.server.listen(qApp.applicationName() + '-' + util.getuser())
-
-    def newConnection(self):
-        socket = self.server.nextPendingConnection()
-        if socket:
-            socket.waitForReadyRead(10000)
-            root = str(socket.readAll())
-            if root and root != '[echo]':
-                self._openRepo(root, reuse=True)
-
-                # Bring the workbench window to the front
-                # This assumes that the client process has
-                # called allowSetForegroundWindow(-1) right before
-                # sending the request
-                self.setWindowState(self.windowState() & ~Qt.WindowMinimized
-                                    | Qt.WindowActive)
-                self.show()
-                self.raise_()
-                self.activateWindow()
-                # Revoke the blanket permission to set the foreground window
-                allowSetForegroundWindow(os.getpid())
-
-            socket.write(QByteArray(root))
-            socket.flush()
-
-def allowSetForegroundWindow(processid=-1):
-    """Allow a given process to set the foreground window"""
-    # processid = -1 means ASFW_ANY (i.e. allow any process)
-    if os.name == 'nt':
-        # on windows we must explicitly allow bringing the main window to
-        # the foreground. To do so we must use ctypes
-        try:
-            from ctypes import windll
-            windll.user32.AllowSetForegroundWindow(processid)
-        except ImportError:
-            pass
-
-def connectToExistingWorkbench(root=None):
-    """
-    Connect and send data to an existing workbench server
-
-    For the connection to be successful, the server must loopback the data
-    that we send to it.
-
-    Normally the data that is sent will be a repository root path, but we can
-    also send "echo" to check that the connection works (i.e. that there is a
-    server)
-    """
-    if root:
-        data = root
-    else:
-        data = '[echo]'
-    socket = QLocalSocket()
-    socket.connectToServer(qApp.applicationName() + '-' + util.getuser(),
-        QIODevice.ReadWrite)
-    if socket.waitForConnected(10000):
-        # Momentarily let any process set the foreground window
-        # The server process with revoke this permission as soon as it gets
-        # the request
-        allowSetForegroundWindow()
-        socket.write(QByteArray(data))
-        socket.flush()
-        socket.waitForReadyRead(10000)
-        reply = socket.readAll()
-        if data == reply:
-            return True
-    return False
-
-def run(ui, *pats, **opts):
-    root = opts.get('root') or paths.find_root()
-    if root and pats:
-        repo = thgrepo.repository(ui, root)
-        pats = hglib.canonpaths(pats)
-        if len(pats) == 1 and os.path.isfile(repo.wjoin(pats[0])):
-            from tortoisehg.hgqt.filedialogs import FileLogDialog
-            fname = pats[0]
-            ufname = hglib.tounicode(fname)
-            dlg = FileLogDialog(repo, fname, None)
-            dlg.setWindowTitle(_('Hg file log viewer [%s] - %s') % (
-                repo.displayname, ufname))
-            return dlg
-
-    # Before starting the workbench, we must check if we must try to reuse an
-    # existing workbench window (we don't by default)
-    # Note that if the "single workbench mode" is enabled, and there is no
-    # existing workbench window, we must tell the Workbench object to create
-    # the workbench server
-    singleworkbenchmode = ui.configbool('tortoisehg', 'workbench.single', True)
-    mustcreateserver = False
-    if singleworkbenchmode:
-        newworkbench = opts.get('newworkbench')
-        if root and not newworkbench:
-            if connectToExistingWorkbench(root):
-                # The were able to connect to an existing workbench server, and
-                # it confirmed that it has opened the selected repo for us
-                sys.exit(0)
-            # there is no pre-existing workbench server
-            serverexists = False
-        else:
-            serverexists = connectToExistingWorkbench('[echo]')
-        # When in " single workbench mode", we must create a server if there
-        # is not one already
-        mustcreateserver = not serverexists
-
-    w = Workbench(createserver=mustcreateserver)
-    if root:
-        root = hglib.tounicode(root)
-        bundle = opts.get('bundle')
-        if bundle:
-            w._openRepo(hglib.fromunicode(root), False,
-                        bundle=hglib.tounicode(bundle))
-        else:
-            w.showRepo(root)
-
-        if pats:
-            q = []
-            for pat in pats:
-                f = repo.wjoin(pat)
-                if os.path.isdir(f):
-                    q.append('file("%s/**")' % pat)
-                elif os.path.isfile(f):
-                    q.append('file("%s")' % pat)
-            w.setRevsetFilter(root, ' or '.join(q))
-    if w.repoTabsWidget.count() <= 0:
-        w.reporegistry.setVisible(True)
-    return w
+    @pyqtSlot()
+    def _editCustomToolsSettings(self):
+        self.editSettings('tools')
